@@ -1278,19 +1278,21 @@ g_string_append_vprintf (GString     *string,
 			 const gchar *fmt,
 			 va_list      args)
 {
-  va_list sneak;
-  gsize length;
-
+  gchar *buf;
+  gint len;
+  
   g_return_if_fail (string != NULL);
   g_return_if_fail (fmt != NULL);
 
-  G_VA_COPY (sneak, args);
-  length = g_printf_string_upper_bound (fmt, sneak);
-  va_end (sneak);
+  len = g_vasprintf (&buf, fmt, args);
 
-  g_string_maybe_expand (string, length);
-  length = g_vsnprintf (string->str + string->len, length, fmt, args);
-  string->len += length;
+  if (len >= 0)
+    {
+      g_string_maybe_expand (string, len);
+      memcpy (string->str + string->len, buf, len + 1);
+      string->len += len;
+      g_free (buf);
+    }
 }
 
 /**
