@@ -87,6 +87,12 @@
 #  include <direct.h>
 #  include <shlobj.h>
    /* older SDK (e.g. msvc 5.0) does not have these*/
+#  ifndef CSIDL_MYMUSIC
+#    define CSIDL_MYMUSIC 13
+#  endif
+#  ifndef CSIDL_MYVIDEO
+#    define CSIDL_MYVIDEO 14
+#  endif
 #  ifndef CSIDL_INTERNET_CACHE
 #    define CSIDL_INTERNET_CACHE 32
 #  endif
@@ -958,6 +964,7 @@ g_get_current_dir (void)
 #else	/* !sun || !HAVE_GETCWD */
   while (max_len < G_MAXULONG / 2)
     {
+      g_free (buffer);
       buffer = g_new (gchar, max_len + 1);
       *buffer = 0;
       dir = getcwd (buffer, max_len);
@@ -965,7 +972,6 @@ g_get_current_dir (void)
       if (dir || errno != ERANGE)
 	break;
 
-      g_free (buffer);
       max_len *= 2;
     }
 #endif	/* !sun || !HAVE_GETCWD */
@@ -1394,8 +1400,6 @@ static  gchar   *g_user_config_dir = NULL;
 static  gchar  **g_system_config_dirs = NULL;
 
 static  gchar  **g_user_special_dirs = NULL;
-static  time_t   g_user_special_dirs_mtime = (time_t) -1;
-static  time_t   g_user_special_dirs_stat_time = (time_t) -1;
 
 /* fifteen minutes of fame for everybody */
 #define G_USER_DIRS_EXPIRE      15 * 60
@@ -1765,9 +1769,28 @@ g_get_real_name (void)
  *
  * Note that in contrast to traditional UNIX tools, this function 
  * prefers <filename>passwd</filename> entries over the <envar>HOME</envar> 
- * environment variable.
+ * environment variable. 
  *
- * Returns: the current user's home directory.
+ * One of the reasons for this decision is that applications in many 
+ * cases need special handling to deal with the case where 
+ * <envar>HOME</envar> is
+ * <simplelist>
+ *   <member>Not owned by the user</member>
+ *   <member>Not writeable</member>
+ *   <member>Not even readable</member>
+ * </simplelist>
+ * Since applications are in general <emphasis>not</emphasis> written 
+ * to deal with these situations it was considered better to make 
+ * g_get_homedir() not pay attention to <envar>HOME</envar> and to 
+ * return the real home directory for the user. If applications
+ * want to pay attention to <envar>HOME</envar>, they can do:
+ * <informalexample><programlisting>
+ *  const char *homedir = g_getenv ("HOME");
+ *   if (!homedir)
+ *      homedir = g_get_homedir (<!-- -->);
+ * </programlisting></informalexample>
+ *
+ * Returns: the current user's home directory
  */
 G_CONST_RETURN gchar*
 g_get_home_dir (void)
@@ -2172,7 +2195,7 @@ static void
 load_user_special_dirs (void)
 {
   g_user_special_dirs[G_USER_DIRECTORY_DESKTOP] = get_special_folder (CSIDL_DESKTOPDIRECTORY);
-  g_user_special_dirs[G_USER_DIRECTORY_DOCUMENTS] = get_special_folder (CSIDL_MYDOCUMENTS);
+  g_user_special_dirs[G_USER_DIRECTORY_DOCUMENTS] = get_special_folder (CSIDL_PERSONAL);
   g_user_special_dirs[G_USER_DIRECTORY_DOWNLOAD] = get_special_folder (CSIDL_DESKTOPDIRECTORY); /* XXX correct ? */
   g_user_special_dirs[G_USER_DIRECTORY_MUSIC] = get_special_folder (CSIDL_MYMUSIC);
   g_user_special_dirs[G_USER_DIRECTORY_PICTURES] = get_special_folder (CSIDL_MYPICTURES);
@@ -2182,78 +2205,10 @@ load_user_special_dirs (void)
 }
 #endif /* G_OS_WIN32 */
 
-#if defined(G_OS_WIN32) || defined(HAVE_CARBON)
-static void 
-maybe_expire_user_special_dirs (void)
-{
-  /* expire the user dirs after G_USER_DIRS_EXPIRE seconds */
-  time_t now;
-
-  time (&now);
-  if (now > g_user_special_dirs_mtime + G_USER_DIRS_EXPIRE &&
-      g_user_special_dirs)
-    {
-      gint i;
-
-      for (i = 0; i < G_USER_N_DIRECTORIES; i++)
-        g_free (g_user_special_dirs[i]);
-
-      g_free (g_user_special_dirs);
-      g_user_special_dirs = NULL;
-    }
-
-  if (g_user_special_dirs == NULL)
-    g_user_special_dirs_mtime = now;
-}
-#endif
+static void g_init_user_config_dir (void);
 
 #if defined(G_OS_UNIX) && !defined(HAVE_CARBON)
 
-/* expire g_user_special_dirs if the config file
- * was modified between different reads
- */
-static void
-maybe_expire_user_special_dirs (void)
-{
-  gchar *config_file;
-  struct stat stat_buf;
-  time_t now;
-
-  /* don't stat() the file more often than necessary */
-  time (&now);
-  if (now < g_user_special_dirs_stat_time + 5)
-    return;
-
-  g_user_special_dirs_stat_time = now;
-
-  config_file = g_build_filename (g_user_config_dir,
-                                  "user-dirs.dirs",
-                                  NULL);
-  
-  if (stat (config_file, &stat_buf) < 0)
-    goto out;
-
-  if (stat_buf.st_mtime != g_user_special_dirs_mtime &&
-      g_user_special_dirs != NULL)
-    {
-      gint i;
-
-      for (i = 0; i < G_USER_N_DIRECTORIES; i++)
-        g_free (g_user_special_dirs[i]);
-
-      g_free (g_user_special_dirs);
-      g_user_special_dirs = NULL;
-    }
-
-    g_user_special_dirs_mtime = stat_buf.st_mtime;
-
-out:
-  g_free (config_file);
-}
-
-
-static void g_init_user_config_dir (void);
-
 /* adapted from xdg-user-dir-lookup.c
  *
  * Copyright (C) 2007 Red Hat Inc.
@@ -2267,7 +2222,7 @@ static void g_init_user_config_dir (void);
  * subject to the following conditions: 
  *
  * The above copyright notice and this permission notice shall be
-/* included in all copies or substantial portions of the Software. 
+ * included in all copies or substantial portions of the Software. 
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
@@ -2418,6 +2373,10 @@ load_user_special_dirs (void)
  *
  * On Unix this is done using the XDG special user directories.
  *
+ * Depending on the platform, the user might be able to change the path
+ * of the special directory without requiring the session to restart; GLib
+ * will not reflect any change once the special directories are loaded.
+ *
  * Return value: the path to the specified special directory, or %NULL
  *   if the logical id was not found. The returned string is owned by
  *   GLib and should not be modified or freed.
@@ -2432,11 +2391,20 @@ g_get_user_special_dir (GUserDirectory directory)
 
   G_LOCK (g_utils_global);
 
-  maybe_expire_user_special_dirs ();
-  if (g_user_special_dirs == NULL)
+  if (G_UNLIKELY (g_user_special_dirs == NULL))
     {
       g_user_special_dirs = g_new0 (gchar *, G_USER_N_DIRECTORIES);
+
       load_user_special_dirs ();
+
+      /* Special-case desktop for historical compatibility */
+      if (g_user_special_dirs[G_USER_DIRECTORY_DESKTOP] == NULL)
+        {
+          g_get_any_init ();
+
+          g_user_special_dirs[G_USER_DIRECTORY_DESKTOP] =
+            g_build_filename (g_home_dir, "Desktop", NULL);
+        }
     }
 
   G_UNLOCK (g_utils_global);
